package com.xylink.sdk;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.xylink.model.HttpMethod;
import com.xylink.util.HttpUtil;
import com.xylink.util.Result;
import org.apache.commons.lang3.StringUtils;

import java.io.IOException;

/**
 * @Classname SdkMethod
 * @Date 2022/4/6 10:39 上午
 * @Created by anan
 * @Description sdkjar 通用方法，用于用户调用SDKserver的接口，用户需要传入url，body以及选择正确的请求方法即可
 * url的形式如下：/api/rest/xxx?xxx=xxxxx
 * 需要事先进行初始化
 * <p>
 * #####################
 * 需要使用签名算法2.0   #
 * ####################
 */
public class SdkMethodV2 extends SdkMethodBase {

    /**
     * 使用签名2.0方法
     *
     * @param url /api/rest/xxx?xxx=xxxxx,不包含&signature=XXX
     * @return 返回文档描述中的对象
     * @throws IOException 抛出异常
     */
    public static <T> Result<T> sdkGet(String url, Class<T> responseType) throws IOException {
        return sdkGetWithClienId(url, null, responseType);
    }
    public static <T> Result<T> sdkGetWithClienId(String url, String clientId, Class<T> responseType) throws IOException {
        url = appendSignAndHost(url, null, StringUtils.EMPTY, HttpMethod.GET.name(),clientId);
        return HttpUtil.getResponse(url, HttpMethod.GET.name(), StringUtils.EMPTY, responseType, clientId);
    }

    /**
     * 签名2.0方法
     *
     * @param url  /api/rest/xxx?xxx=xxxxx,不包含&signature=XXX
     * @param body 文档描述中的body内容
     * @return 返回文档描述中的对象
     * @throws IOException 抛出异常
     */
    public static <T> Result<T> sdkPost(String url, Object body, Class<T> responseType) throws IOException {
        return sdkPostWithClienId(url, null, body, responseType);
    }
    public static <T> Result<T> sdkPostWithClienId(String url, String clientId, Object body, Class<T> responseType) throws IOException {
        //body序列化
        String jsonEntity = new ObjectMapper().writeValueAsString(body);
        url = appendSignAndHost(url, null, jsonEntity, HttpMethod.POST.name(), clientId);
        return HttpUtil.getResponse(url, HttpMethod.POST.name(), jsonEntity, responseType, clientId);
    }

    /**
     * 签名2.0方法
     *
     * @param url  /api/rest/xxx?xxx=xxxxx,不包含&signature=XXX
     * @param body 文档描述中的body内容
     * @return 返回文档描述中的对象
     * @throws IOException 抛出异常
     */
    public static <T> Result<T> sdkPut(String url, Object body, Class<T> responseType) throws IOException {
        return sdkPutWithClienId(url, null, body, responseType);
    }

    public static <T> Result<T> sdkPutWithClienId(String url, String clientId, Object body, Class<T> responseType) throws IOException {
        //body序列化
        String jsonEntity = new ObjectMapper().writeValueAsString(body);
        url = appendSignAndHost(url, null, jsonEntity, HttpMethod.PUT.name(), clientId);
        return HttpUtil.getResponse(url, HttpMethod.PUT.name(), jsonEntity, responseType, clientId);
    }

    /**
     * 签名2.0
     *
     * @param url /api/rest/xxx?xxx=xxxxx,不包含&signature=XXX
     * @return 返回文档描述中的对象
     * @throws IOException 抛出异常
     */
    public static <T> Result<T> sdkDelete(String url, Class<T> responseType) throws IOException {
        return sdkDeleteWithClienId(url, null, responseType);
    }
    public static <T> Result<T> sdkDeleteWithClienId(String url, String clientId, Class<T> responseType) throws IOException {
        url = appendSignAndHost(url, null, StringUtils.EMPTY, HttpMethod.DELETE.name(), clientId);
        return HttpUtil.getResponse(url, HttpMethod.DELETE.name(), StringUtils.EMPTY, responseType, clientId);
    }
}
